# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****


"""

Utilities for authentication via a generic user backend.

Include mozsvc.user into your pyramid config to get the following niceties:

    * user database backend loaded as a plugin from "auth" config section
    * user database backend available as request.registry["auth"]
    * authenticated user's data as a dict at request.user

You can also use the function mozsvc.user.authenticate(req, creds) as a
shortcut for authenticating against the configured backend, which is useful
for writing your own authentication plugins.

This module is also designed to "play nice" with authentication schemes using
repoze.who and the pyramid_whoauth package.  For example, the request.user
attribute actually just exposes environ["repoze.who.identity"].  For some
additional integration with pyramid_whoauth, see the mozsvc.user.whoauth
module.

"""

from pyramid.request import Request
from pyramid.security import authenticated_userid

from cef import log_cef, AUTH_FAILURE

from mozsvc.plugin import load_and_register

import logging
logger = logging.getLogger("mozsvc.user")


class RequestWithUser(Request):
    """Request object that exposes the current user as "request.user".

    This is a custom Pyramid Request class that does user authentication
    on demand.  When you access the "user" attribute it will automatically
    search for credentials in the request, authenticate them, create a dict
    of user data and return it.

    If there are no credentials in the request then the "user" attribute will
    be an empty dict.  If the call to authenticated_userid() raises an error
    it will *not* be intercepted, you will receive it as an error from the
    attribute access attempt.

    This class also provides some conveniences for integrating with repoze.who,
    by storing the registry and user object in the WSGI environ rather than on
    the request object itself.  This lets us write repoze.who plugins that can
    access the higher-level pyramid config data.
    """

    # Expose the "repoze.who.identity" dict as request.user.
    # This allows for convenient integration between pyramid-level code
    # and repoze.who plugins.

    def _get_user(self):
        # Return an existing identity if there is one.
        user = self.environ.get("repoze.who.identity")
        if user is not None:
            return user

        # Return an under-construction identity if there is one.
        # This lets pyramid-level auth plugins store stuff in req.user.
        user = self.environ.get("mozsvc.user.temp_user")
        if user is not None:
            return user

        # Otherwise, we need to authenticate.
        # Do it through the standard pyramid interface, while providing an
        # under-construction user dict for plugins to scribble on.
        extra = self.environ["mozsvc.user.temp_user"] = {}
        try:
            username = authenticated_userid(self)
        finally:
            self.environ.pop("mozsvc.user.temp_user", None)

        # Now that we've authed, cached the result as repoze.who.identity.
        # For a successful auth it might already exist.  For a failed auth
        # we set it to the empty dict.
        user = self.environ.get("repoze.who.identity")
        if user is None:
            user = self.environ["repoze.who.identity"] = {}

        # If the auth was successful, make sure the identity contains
        # all the expected keys.
        if username is not None:
            user.update(extra)
            user.setdefault("username", username)
            user.setdefault("repoze.who.userid", username)
        return user

    def _set_user(self, user):
        self.environ["repoze.who.identity"] = user

    user = property(_get_user, _set_user)

    # Store the pyramid application registry in the WSGI environ.
    # This allows repoze.who plugins to access pyramid config despite
    # the fact that they aren't passed a request object.

    def _get_registry(self):
        try:
            return self.environ["mozsvc.user.registry"]
        except KeyError:
            raise AttributeError("registry")

    def _set_registry(self, registry):
        self.environ["mozsvc.user.registry"] = registry
        self.__dict__["registry"] = registry

    registry = property(_get_registry, _set_registry)


def authenticate(request, credentials, attrs=()):
    """Authenticate a dict of credentials against the configured user backend.

    This is a handy callback that you can use to check a dict of credentials
    against the configured auth backend.  It will accept credentials from any
    of the auth schemes supported by the backend.  If the authentication is
    successful it will update request.user with the user object loaded from
    the backend.
    """
    # Use whatever auth backend has been configured.
    auth = request.registry.get("auth")
    if auth is None:
        return False

    # Update an existing user object if one exists on the request.
    user = getattr(request, "user", None)
    if user is None:
        user = {}

    # Ensure that we have credentials["username"] for use by the backend.
    # Some repoze.who plugins like to use "login" instead of "username".
    if "username" not in credentials:
        if "login" in credentials:
            credentials["username"] = credentials.pop("login")
        else:
            log_cef("Authentication attemped without username", 5,
                    request.environ, request.registry.settings,
                    "", signature=AUTH_FAILURE)
            return False

    # Normalize the password, if any, to be unicode.
    password = credentials.get("password")
    if password is not None and not isinstance(password, unicode):
        try:
            credentials["password"] = password.decode("utf8")
        except UnicodeDecodeError:
            return None

    # Authenticate against the configured backend.
    if not auth.authenticate_user(user, credentials, attrs):
        log_cef("Authentication Failed", 5,
                request.environ, request.registry.settings,
                credentials["username"],
                signature=AUTH_FAILURE)
        return False

    # Store the user dict on the request, and return it for conveience.
    if getattr(request, "user", None) is None:
        request.user = user
    return user


def includeme(config):
    """Include mozsvc user-handling into a pyramid config.

    This function will set up user-handling via the mozsvc.user system.
    Things configured include:

        * use RequestWithUser as the request object factory
        * load a user database backend from config section "auth"

    """
    config.set_request_factory(RequestWithUser)
    try:
        config.registry["auth"] = load_and_register("auth", config)
    except Exception, e:
        logger.warning("Unable to load auth backend. Problem? %s" % e)
        config.registry["auth"] = None



# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****


"""
Plugin loading system using zope.interface and the pyramid registry.

This module provides a simple plugin-loading system on top of pyramid's
internal component registry.  Suppose you have a plugin interface declared
like so::

    from zope.interface import Interface

    class IMyPlugin(Interface):
        pass

And a concrete implementation of it in "mymodule.MyPluginImpl"::

    from zope.interface import implements

    class MyPluginImpl(object):
        implements(IMyPlugin)
        def __init__(self, foo, bar):
            pass

You can specify a section in your config file like this::

    [plugin1]
    backend = mymodule.MyPluginImpl
    foo = frob
    bar = baz

Then when you come to initialize your Pyramid app configuration, you
just ask it to load and register your plugin like so::

    from mozsvc.plugin import load_and_register

    config = Configurator(...etc...)
    plugin = load_and_register("plugin1", config)

This will read the configuration to find the "backend" declaration and resolve
it to a class, parse each of the other configuration options to use as keyword
arguments, and create an instance of the class for you.

The load_and_register function also registers the loaded plugin with pyramid's
component registry.  This means you can look it up in any other part of your
code by doing the following:

    plugin = request.registry.queryUtility(IMyPlugin)

If you're not interested in this and want to skip all the zope interface
palaver, you can also use *just* the plugin loading functionality by using::

    plugin = load_from_config(section_name, config_obj)

or::

    plugin = load_from_settings(section_name, settings_dict)

"""

from zope.interface import providedBy

from mozsvc.util import resolve_name


def load_and_register(section_name, config, interface=None, registry_name=u""):
    """Load a plugin from the named configuration section.

    Given a Pyramid Configurator object and the name of a config section,
    this function loads the plugin as specified in that section and then
    registers it into the Pyramid registry as a utility for each of the
    interfaces that it provides.
    """
    settings = config.registry.settings
    # Load using the Config object if available, since it's faster.
    # If not then fall back to loading from dotted setting names.
    if "config" in settings:
        plugin = load_from_config(section_name, settings["config"])
    else:
        plugin = load_from_settings(section_name, settings)
    if interface is not None:
        interfaces = [interface]
    else:
        interfaces = providedBy(plugin)
    # Register the plugin for each interface that it provides.
    # Use the Configurators delayed-registration machinery to get
    # conflict-resolution and so-forth for free.
    for interface in interfaces:

        def register(interface=interface):
            config.registry.registerUtility(plugin, interface, registry_name)

        config.action((interface, registry_name), register)
    # And return it for user convenience.
    return plugin


def load_from_config(section_name, config):
    """Load the plugin from the given section in a Config object.

    This function loads a plugin using the settings specified in a Config
    object section.  The key "backend" must be present in the section and
    gives the dotted name of the plugin class to load.  Any other keys in
    the secton will be passed as keyword arguments to the class.
    """
    kwargs = dict(config.items(section_name))
    klass = resolve_name(kwargs.pop("backend"))
    return klass(**kwargs)


def load_from_settings(section_name, settings):
    """Load the plugin from the given section in a settings dict.

    This function loads a plugin using prefixed settings from the pyramid
    settings dict.  Any keys in the settings dict that start with the
    given section name plus a dot will be used.

    This a compatability function for use when a Config object is not
    available; load_from_config will usually be faster.
    """
    kwargs = {}
    prefix = section_name + "."
    for name, value in settings.iteritems():
        if name.startswith(prefix):
            kwargs[name[len(prefix):]] = value
    klass = resolve_name(kwargs.pop("backend"))
    return klass(**kwargs)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import sys
import unittest2
import urlparse

from pyramid.request import Request
from pyramid.interfaces import IRequestFactory

from webtest import TestApp
from wsgiproxy.app import WSGIProxyApp

from mozsvc.config import get_configurator

DEFAULT_SETTINGS = {
    'cef.vendor': 'mozilla',
    'cef.device_version': '1.3',
    'cef.product': 'test',
    'cef.use': True,
    'cef.version': 0,
    'cef.file': 'syslog',
}


def get_test_configurator(root, ini_file="tests.ini"):
    """Find a file with testing settings, turn it into a configurator."""
    ini_dir = root
    while True:
        ini_path = os.path.join(ini_dir, ini_file)
        if os.path.exists(ini_path):
            break
        if ini_path == ini_file or ini_path == "/" + ini_file:
            raise RuntimeError("cannot locate " + ini_file)
        ini_dir = os.path.split(ini_dir)[0]

    config = get_configurator({"__file__": ini_path}, **DEFAULT_SETTINGS)
    return config


# Try to convince test-loading tools to ignore this function
# despite the fact that it has "test" in the name.
get_test_configurator.__test__ = False


def make_request(config, path="/", environ=None, factory=None):
    my_environ = {}
    my_environ["wsgi.version"] = "1.0"
    my_environ["REQUEST_METHOD"] = "GET"
    my_environ["SCRIPT_NAME"] = ""
    my_environ["PATH_INFO"] = path
    my_environ["SERVER_NAME"] = "localhost"
    my_environ["SERVER_PORT"] = "5000"
    if environ is not None:
        my_environ.update(environ)
    if factory is None:
        factory = config.registry.queryUtility(IRequestFactory)
        if factory is None:
            factory = Request
    request = factory(my_environ)
    request.registry = config.registry
    return request


class TestCase(unittest2.TestCase):
    """TestCase with some generic helper methods."""

    def setUp(self):
        super(TestCase, self).setUp()
        self.config = self.get_configurator()

    def tearDown(self):
        self.config.end()
        super(TestCase, self).tearDown()

    def get_configurator(self):
        """Load the configurator to use for the tests."""
        # Load config from the .ini file.
        # The file to use may be specified in the environment.
        self.ini_file = os.environ.get("MOZSVC_TEST_INI_FILE", "tests.ini")
        __file__ = sys.modules[self.__class__.__module__].__file__
        config = get_test_configurator(__file__, self.ini_file)
        config.begin()
        return config

    def make_request(self, *args, **kwds):
        return make_request(self.config, *args, **kwds)


class FunctionalTestCase(TestCase):
    """TestCase for writing functional tests using WebTest.

    This TestCase subclass provides an easy mechanism to write functional
    tests using WebTest.  It exposes a TestApp instance as self.app.

    If the environment variable MOZSVC_TEST_REMOTE is set to a URL, then
    self.app will be a WSGIProxy application that forwards all requests to
    that server.  This allows the functional tests to be easily run against
    a live server instance.
    """

    def setUp(self):
        super(FunctionalTestCase, self).setUp()

        # Test against a live server if instructed so by the environment.
        # Otherwise, test against an in-process WSGI application.
        test_remote = os.environ.get("MOZSVC_TEST_REMOTE")
        if not test_remote:
            self.distant = False
            self.host_url = "http://localhost:5000"
            application = self.config.make_wsgi_app()
        else:
            self.distant = True
            self.host_url = test_remote
            application = WSGIProxyApp(test_remote)

        host_url = urlparse.urlparse(self.host_url)
        self.app = TestApp(application, extra_environ={
            "HTTP_HOST": host_url.netloc,
            "wsgi.url_scheme": host_url.scheme or "http",
            "SERVER_NAME": host_url.hostname,
            "REMOTE_ADDR": "127.0.0.1",
        })



# -*- coding: utf-8 -*-

# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****

import unittest
import os.path

from mozsvc.util import round_time, resolve_name, maybe_resolve_name


class TestUtil(unittest.TestCase):

    def test_round_time(self):

        # returns a two-digits decimal of the current time
        res = round_time()
        self.assertEqual(len(str(res).split('.')[-1]), 2)

        # can take a timestamp
        res = round_time(129084.198271987)
        self.assertEqual(str(res), '129084.20')

        # can take a str timestamp
        res = round_time('129084.198271987')
        self.assertEqual(str(res), '129084.20')

        # bad values raise ValueErrors
        self.assertRaises(ValueError, round_time, 'bleh')
        self.assertRaises(ValueError, round_time, object())

        # changing the precision
        res = round_time(129084.198271987, precision=3)
        self.assertEqual(str(res), '129084.198')

    def test_resolve_name(self):

        # Resolving by absolute path
        self.assertEquals(os.path.abspath, resolve_name("os.path.abspath"))
        self.assertEquals(os.path.abspath, resolve_name("os.path:abspath"))

        # Resolving by relative path to package object
        self.assertEquals(os.path.abspath, resolve_name(".path.abspath", os))
        self.assertEquals(os.path.abspath, resolve_name(".path:abspath", os))

        # Resolving by relative path to package name
        self.assertEquals(os.path.abspath, resolve_name(".abspath", "os.path"))
        self.assertEquals(os.path.abspath, resolve_name(":abspath", "os.path"))

    def test_maybe_resolve_name(self):

        self.assertEquals(os.path, maybe_resolve_name("os.path"))
        self.assertEquals(os.path, maybe_resolve_name(os.path))
        self.assertEquals(None, maybe_resolve_name(None))



# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****

import unittest
import tempfile

from zope.interface import implements

import pyramid.testing
import pyramid.request
from pyramid.interfaces import IRequestFactory, IAuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy
from pyramid.security import authenticated_userid
from pyramid.request import Request

import tokenlib
import macauthlib

import mozsvc.user


class HeaderAuthenticationPolicy(object):
    """Authentication policy taking creds from request headers."""

    implements(IAuthenticationPolicy)

    def authenticated_userid(self, request):
        username = request.environ.get("HTTP_X_USERNAME")
        if not username:
            return None
        password = request.environ.get("HTTP_X_PASSWORD")
        if not password:
            return None
        credentials = {"username": username, "password": password}
        if not mozsvc.user.authenticate(request, credentials, ["mail"]):
            return None
        request.user["x-was-ere"] = True
        return username

    def unauthenticated_userid(self, request):
        raise RuntimeError("tests shouldn't call this")  # pragma: nocover

    def effective_principals(self, request):
        raise RuntimeError("tests shouldn't call this")  # pragma: nocover


class TestCaseHelpers(object):

    DEFAULT_SETTINGS = {
        'cef.vendor': 'mozilla',
        'cef.device_version': '1.3',
        'cef.product': 'weave',
        'cef.use': True,
        'cef.version': 0,
        'cef.file': 'syslog',
    }

    def setUp(self):
        self.config = pyramid.testing.setUp()
        self.config.add_settings(self.DEFAULT_SETTINGS)

    def tearDown(self):
        pyramid.testing.tearDown()

    def _make_request(self, environ=None, factory=None, config=None):
        my_environ = {}
        my_environ["wsgi.version"] = "1.0"
        my_environ["wsgi.url_scheme"] = "http"
        my_environ["REQUEST_METHOD"] = "GET"
        my_environ["SCRIPT_NAME"] = ""
        my_environ["PATH_INFO"] = "/"
        my_environ["SERVER_NAME"] = "localhost"
        my_environ["SERVER_PORT"] = "80"
        if environ is not None:
            my_environ.update(environ)
        if config is None:
            config = self.config
        if factory is None:
            factory = config.registry.getUtility(IRequestFactory)
        request = factory(my_environ)
        request.registry = config.registry
        return request


class FakeAuthBackend(object):
    """Auth backend to use for running the tests."""

    def __init__(self):
        self.users = {}

    def create_user(self, username, password, email):
        self.users[username] = [password, email]

    def authenticate_user(self, user, credentials, attrs=None):
        try:
            username = credentials.get("username", user.get("username"))
            if credentials["password"] == self.users[username][0]:
                user.setdefault("username", username)
                user.setdefault("mail", self.users[username][1])
                return username
        except KeyError:
            pass
        return None

    def update_password(self, user, credentials, new_password):
        username = self.authenticate_user(user, credentials)
        if not username:
            return False
        self.users[username][0] = new_password


class UserTestCase(TestCaseHelpers, unittest.TestCase):

    DEFAULT_SETTINGS = TestCaseHelpers.DEFAULT_SETTINGS.copy()
    DEFAULT_SETTINGS.update({
        "auth.backend": "mozsvc.tests.test_user:FakeAuthBackend",
    })

    def setUp(self):
        super(UserTestCase, self).setUp()
        self.config.include("mozsvc.user")
        self.config.set_authorization_policy(ACLAuthorizationPolicy())
        self.config.set_authentication_policy(HeaderAuthenticationPolicy())
        self.auth = self.config.registry["auth"]
        self.auth.create_user("user1", "password1", "test@mozilla.com")

    def test_auth_backend_is_loaded(self):
        self.assertEquals(self.config.registry["auth"].__class__.__name__,
                          "FakeAuthBackend")

    def test_authenticate(self):
        request = self._make_request()
        # We have no IAuthenticationPolicy, so initially req.user is False.
        self.assertFalse(request.user)
        # After authenticating, it will be replaced with the user object.
        credentials = {"username": "user1", "password": "password1"}
        mozsvc.user.authenticate(request, credentials)
        self.assertEquals(request.user["username"], "user1")

    def test_authenticate_with_normal_request_object(self):
        request = self._make_request(factory=Request)
        # Initially it has no user attribute.
        self.assertRaises(AttributeError, getattr, request, "user")
        # After authenticating, it will have the user object.
        credentials = {"username": "user1", "password": "password1"}
        mozsvc.user.authenticate(request, credentials)
        self.assertEquals(request.user["username"], "user1")

    def test_authenticate_with_no_backend(self):
        del self.config.registry["auth"]
        request = self._make_request()
        credentials = {"username": "user1", "password": "password1"}
        self.assertFalse(mozsvc.user.authenticate(request, credentials))
        self.assertEquals(request.user, {})

    def test_authenticate_with_repozewho_style_credentials(self):
        request = self._make_request()
        credentials = {"login": "user1", "password": "password1"}
        mozsvc.user.authenticate(request, credentials)
        self.assertEquals(request.user["username"], "user1")

    def test_authenticate_with_bad_password(self):
        request = self._make_request()
        credentials = {"username": "user1", "password": "BAD BAD BAD"}
        self.assertFalse(mozsvc.user.authenticate(request, credentials))
        self.assertEquals(request.user, {})

    def test_authenticate_with_unicode_password(self):
        credentials = {"username": "user1", "password": "password1"}
        new_password = u"password\N{GREEK SMALL LETTER ALPHA}"
        self.auth.update_password({"username": "user1"}, credentials,
                                  new_password)
        # Auth works with unicode password.
        request = self._make_request()
        credentials = {"username": "user1", "password": new_password}
        self.assertTrue(mozsvc.user.authenticate(request, credentials))
        # Auth works with utf-encoded password.
        request = self._make_request()
        credentials = {"username": "user1",
                       "password": new_password.encode("utf8")}
        self.assertTrue(mozsvc.user.authenticate(request, credentials))
        # Auth fails with badly-encoded password
        request = self._make_request()
        credentials = {"username": "user1",
                       "password": new_password.encode("utf16")}
        self.assertFalse(mozsvc.user.authenticate(request, credentials))

    def test_authenticate_with_unknown_username(self):
        request = self._make_request()
        credentials = {"username": "user2", "password": "password1"}
        self.assertFalse(mozsvc.user.authenticate(request, credentials))
        self.assertEquals(request.user, {})

    def test_authenticate_with_no_username(self):
        request = self._make_request()
        credentials = {"usernme": "user2", "password": "password1"}
        self.assertFalse(mozsvc.user.authenticate(request, credentials))
        self.assertEquals(request.user, {})

    def test_includeme_with_bad_backend(self):
        config = pyramid.testing.setUp()
        self.config.add_settings({
            "auth.backend": "this.does.not.exist",
        })
        config.include("mozsvc.user")
        self.assertEquals(config.registry["auth"], None)
        request = self._make_request()
        self.assertEquals(request.user, {})

    def test_req_user_success(self):
        request = self._make_request({
                    "HTTP_X_USERNAME": "user1",
                    "HTTP_X_PASSWORD": "password1",
                  })
        self.assertEquals(request.user["username"], "user1")
        self.assertEquals(request.user.get("password"), None)
        self.assertEquals(request.user["mail"], "test@mozilla.com")
        self.assertEquals(request.user["x-was-ere"], True)

    def test_req_user_bad_password(self):
        request = self._make_request({
                    "HTTP_X_USERNAME": "user1",
                    "HTTP_X_PASSWORD": "random_guess",
                  })
        self.assertFalse(request.user)

    def test_req_user_no_username(self):
        request = self._make_request({
                    "HTTP_X_PASSWORD": "password1",
                  })
        self.assertFalse(request.user)

    def test_req_user_no_password(self):
        request = self._make_request({
                    "HTTP_X_USERNAME": "user1",
                  })
        self.assertFalse(request.user)

    def test_req_user_exposes_repoze_who_identity(self):
        # An existing r.w.i dict is exposed as req.user.
        request = self._make_request({
                    "repoze.who.identity": {"repoze-was-ere": True},
                  })
        self.assertEquals(request.user["repoze-was-ere"], True)
        # Setting a key in req.user also sets it in r.w.i.
        request.user["testing"] = "testing"
        self.assertEquals(request.environ["repoze.who.identity"]["testing"],
                          "testing")
        # Replacing req.user also replaces r.w.i
        request.user = {"replacement": "text"}
        self.assertEquals(request.environ["repoze.who.identity"].keys(),
                          ["replacement"])

    def test_registry_is_stored_in_environment(self):
        request = self._make_request()
        self.assertEquals(self.config.registry, request.registry)
        self.assertEquals(self.config.registry,
                          request.environ["mozsvc.user.registry"])
        del request.environ["mozsvc.user.registry"]
        self.assertRaises(AttributeError, getattr, request, "registry")


class FakeAuthPlugin(object):
    def authenticate(self, environ, identity):
        raise RuntimeError("should not run")  # pragma: nocover


class FakeIdentifierPlugin(object):
    def identify(self, environ, identity):
        raise RuntimeError("should not run")  # pragma: nocover


class UserWhoAuthTestCase(TestCaseHelpers, unittest.TestCase):

    DEFAULT_SETTINGS = TestCaseHelpers.DEFAULT_SETTINGS.copy()
    DEFAULT_SETTINGS.update({
        "auth.backend": "mozsvc.tests.test_user:FakeAuthBackend",
    })

    def setUp(self):
        super(UserWhoAuthTestCase, self).setUp()
        self.config.include("mozsvc.user.whoauth")
        self.auth = self.config.registry["auth"]
        self.auth.create_user("user1", "password1", "test@mozilla.com")

    def test_that_basic_auth_is_used_by_default(self):
        # With no who-specific settings, we get a backend authenticator
        # and use basic-auth for identification and challenge.
        authz = "Basic " + "user1:password1".encode("base64").strip()
        req = self._make_request({
            "HTTP_AUTHORIZATION": authz
        })
        self.assertEquals(authenticated_userid(req), "user1")
        authz = "Basic " + "user1:WRONG".encode("base64").strip()
        req = self._make_request({
            "HTTP_AUTHORIZATION": authz
        })
        self.assertEquals(authenticated_userid(req), None)

    def test_that_macauth_is_used_by_default(self):
        # Grab the MACAuth plugin so we can sign requests.
        policy = self.config.registry.queryUtility(IAuthenticationPolicy)
        api_factory = policy.api_factory
        mac_plugin = api_factory.authenticators[1][1]
        # Generate signed request.
        req = self._make_request()
        id, key = mac_plugin.encode_mac_id(req, {"uid": 42})
        macauthlib.sign_request(req, id, key)
        # That should be enough to authenticate.
        self.assertEquals(authenticated_userid(req), 42)
        # Check that it rejects invalid MAC ids.
        req = self._make_request()
        macauthlib.sign_request(req, id, key)
        authz = req.environ["HTTP_AUTHORIZATION"]
        req.environ["HTTP_AUTHORIZATION"] = authz.replace(id, "XXX" + id)
        self.assertRaises(Exception, authenticated_userid, req)

    def test_that_basic_auth_is_not_used_when_theres_no_backend(self):
        config2 = pyramid.testing.setUp()
        config2.add_settings(self.DEFAULT_SETTINGS)
        del config2.registry.settings["auth.backend"]
        config2.include("mozsvc.user.whoauth")
        config2.commit()
        # There should be no backend.
        self.assertEquals(config2.registry.get("auth"), None)
        # The policy should have only macauth.
        policy = config2.registry.queryUtility(IAuthenticationPolicy)
        api_factory = policy.api_factory
        self.assertEquals(len(api_factory.authenticators), 1)
        self.assertEquals(api_factory.authenticators[0][1].__class__.__name__,
                          "SagradaMACAuthPlugin")
        self.assertEquals(len(api_factory.identifiers), 1)
        self.assertEquals(api_factory.identifiers[0][1].__class__.__name__,
                          "SagradaMACAuthPlugin")

    def test_that_explicit_settings_are_not_overridden(self):
        # Create a new config with some explicit who-auth settings.
        config2 = pyramid.testing.setUp()
        config2.add_settings(self.DEFAULT_SETTINGS)
        config2.add_settings({
            "who.authenticators.plugins": "fakeauth",
            "who.identifiers.plugins": "fakeid",
            "who.challengers.plugins": "fakeid",
            "who.plugin.fakeauth.use":
                "mozsvc.tests.test_user:FakeAuthPlugin",
            "who.plugin.fakeid.use":
                "mozsvc.tests.test_user:FakeIdentifierPlugin",
        })
        config2.include("mozsvc.user.whoauth")
        config2.commit()
        # Now poke at it to see what has been overridden.
        policy = config2.registry.queryUtility(IAuthenticationPolicy)
        api_factory = policy.api_factory
        self.assertEquals(len(api_factory.authenticators), 1)
        self.assertEquals(api_factory.authenticators[0][1].__class__.__name__,
                          "FakeAuthPlugin")
        self.assertEquals(len(api_factory.identifiers), 1)
        self.assertEquals(api_factory.identifiers[0][1].__class__.__name__,
                          "FakeIdentifierPlugin")

    def test_graceful_handling_of_bad_auth_policy(self):
        # This just re-tests the functionality from mozvsc.user, making
        # sure that a non-repoze-who policy doesnt mess it up.
        config2 = pyramid.testing.setUp(autocommit=False)
        config2.add_settings(self.DEFAULT_SETTINGS)
        config2.include("mozsvc.user.whoauth")
        config2.set_authentication_policy(HeaderAuthenticationPolicy())
        config2.commit()
        request = self._make_request(factory=mozsvc.user.RequestWithUser)
        self.assertFalse(request.user)
        credentials = {"username": "user1", "password": "password1"}
        mozsvc.user.authenticate(request, credentials)
        self.assertEquals(request.user["username"], "user1")

    def test_graceful_handling_of_other_request_objects(self):
        authz = "Basic " + "user1:password1".encode("base64").strip()
        req = self._make_request({
            "HTTP_AUTHORIZATION": authz
        }, factory=pyramid.request.Request)
        # This makes the request available via get_current_request()
        self.config.begin(request=req)
        try:
            self.assertEquals(authenticated_userid(req), "user1")
        finally:
            self.config.end()

    def test_that_macauth_cant_use_both_secret_and_secrets_file(self):
        config2 = pyramid.testing.setUp()
        config2.add_settings(self.DEFAULT_SETTINGS)
        config2.add_settings({
            "who.plugin.macauth.secret": "DARTH VADER IS LUKE'S FATHER",
            "who.plugin.macauth.secrets_file": "/dev/null",
        })
        self.assertRaises(ValueError, config2.include, "mozsvc.user.whoauth")

    def test_that_macauth_can_use_per_node_hostname_secrets(self):
        with tempfile.NamedTemporaryFile() as sf:
            # Write some secrets to a file.
            sf.write("http://host1.com,0001:secret11,0002:secret12\n")
            sf.write("https://host2.com,0001:secret21,0002:secret22\n")
            sf.write("https://host3.com:444,0001:secret31,0002:secret32\n")
            sf.flush()
            # Configure the plugin to load them.
            config2 = pyramid.testing.setUp()
            config2.add_settings(self.DEFAULT_SETTINGS)
            config2.add_settings({
                "who.plugin.macauth.secrets_file": sf.name,
            })
            config2.include("mozsvc.user.whoauth")
            config2.commit()
            # It should accept a request signed with the old secret on host1.
            req = self._make_request(config=config2, environ={
                "HTTP_HOST": "host1.com",
            })
            id = tokenlib.make_token({"uid": 42}, secret="secret11")
            key = tokenlib.get_token_secret(id, secret="secret11")
            macauthlib.sign_request(req, id, key)
            self.assertEquals(authenticated_userid(req), 42)
            # It should accept a request signed with the new secret on host1.
            req = self._make_request(config=config2, environ={
                "HTTP_HOST": "host1.com",
            })
            id = tokenlib.make_token({"uid": 42}, secret="secret12")
            key = tokenlib.get_token_secret(id, secret="secret12")
            macauthlib.sign_request(req, id, key)
            self.assertEquals(authenticated_userid(req), 42)
            # It should reject a request signed with secret from other host.
            req = self._make_request(config=config2, environ={
                "HTTP_HOST": "host2.com",
            })
            id = tokenlib.make_token({"uid": 42}, secret="secret12")
            key = tokenlib.get_token_secret(id, secret="secret12")
            macauthlib.sign_request(req, id, key)
            self.assertRaises(Exception, authenticated_userid, req)
            # It should reject a request over plain http when host2 is ssl.
            req = self._make_request(config=config2, environ={
                "HTTP_HOST": "host2.com",
            })
            id = tokenlib.make_token({"uid": 42}, secret="secret22")
            key = tokenlib.get_token_secret(id, secret="secret22")
            macauthlib.sign_request(req, id, key)
            self.assertRaises(Exception, authenticated_userid, req)
            # It should accept a request signed with the new secret on host2.
            req = self._make_request(config=config2, environ={
                "HTTP_HOST": "host2.com",
                "wsgi.url_scheme": "https",
            })
            id = tokenlib.make_token({"uid": 42}, secret="secret22")
            key = tokenlib.get_token_secret(id, secret="secret22")
            macauthlib.sign_request(req, id, key)
            self.assertEquals(authenticated_userid(req), 42)
            # It should accept a request to host2 with an explicit port number.
            req = self._make_request(config=config2, environ={
                "HTTP_HOST": "host2.com:443",
                "wsgi.url_scheme": "https",
            })
            id = tokenlib.make_token({"uid": 42}, secret="secret22")
            key = tokenlib.get_token_secret(id, secret="secret22")
            macauthlib.sign_request(req, id, key)
            self.assertEquals(authenticated_userid(req), 42)
            # It should accept a request to host3 on a custom port.
            req = self._make_request(config=config2, environ={
                "HTTP_HOST": "host3.com:444",
                "wsgi.url_scheme": "https",
            })
            id = tokenlib.make_token({"uid": 42}, secret="secret32")
            key = tokenlib.get_token_secret(id, secret="secret32")
            macauthlib.sign_request(req, id, key)
            self.assertEquals(authenticated_userid(req), 42)
            # It should reject unknown hostnames.
            req = self._make_request(config=config2, environ={
                "HTTP_HOST": "host4.com",
            })
            id = tokenlib.make_token({"uid": 42}, secret="secret12")
            key = tokenlib.get_token_secret(id, secret="secret12")
            macauthlib.sign_request(req, id, key)
            self.assertRaises(Exception, authenticated_userid, req)



# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****

import unittest
import tempfile
import os
from StringIO import StringIO

from mozsvc.exceptions import EnvironmentNotFoundError
from mozsvc.config import (Config, SettingsDict, load_into_settings,
                           get_configurator)


_FILE_ONE = """\
[DEFAULT]
extends = %s

[one]
foo = bar
num = -12
st = "o=k"
lines = 1
        two
        3

env = some ${__STUFF__}

[two]
a = b
"""

_FILE_TWO = """\
[one]
foo = baz
two = "a"

[three]
more = stuff
"""

_FILE_THREE = """\
[DEFAULT]
extends = no-no,no-no-no-no,no-no-no-no,theresnolimit

[one]
foo = bar
"""

_FILE_FOUR = """\
[global]
foo = bar
baz = bawlp

[auth]
a = b
c = d

[storage]
e = f
g = h

[multi:once]
storage.i = j
storage.k = l

[multi:thrice]
storage.i = jjj
storage.k = lll
"""

_EXTRA = """\
[some]
stuff = True

[other]
thing = ok
"""


class ConfigTestCase(unittest.TestCase):

    def setUp(self):
        os.environ['__STUFF__'] = 'stuff'
        fp, filename = tempfile.mkstemp()
        f = os.fdopen(fp, 'w')
        f.write(_FILE_TWO)
        f.close()
        self.file_one = StringIO(_FILE_ONE % filename)
        self.file_two = filename
        self.file_three = StringIO(_FILE_THREE)

        fp, filename = tempfile.mkstemp()
        f = os.fdopen(fp, 'w')
        f.write(_FILE_FOUR)
        f.close()
        self.file_four = filename

    def tearDown(self):
        if '__STUFF__' in os.environ:
            del os.environ['__STUFF__']
        os.remove(self.file_two)

    def test_reader(self):
        config = Config(self.file_one)

        # values conversion
        self.assertEquals(config.get('one', 'foo'), 'bar')
        self.assertEquals(config.get('one', 'num'), -12)
        self.assertEquals(config.get('one', 'st'), 'o=k')
        self.assertEquals(config.get('one', 'lines'), [1, 'two', 3])
        self.assertEquals(config.get('one', 'env'), 'some stuff')

        # getting a map
        map = config.get_map()
        self.assertEquals(map['one.foo'], 'bar')

        map = config.get_map('one')
        self.assertEquals(map['foo'], 'bar')

        del os.environ['__STUFF__']
        self.assertRaises(EnvironmentNotFoundError, config.get, 'one', 'env')

        # extends
        self.assertEquals(config.get('three', 'more'), 'stuff')
        self.assertEquals(config.get('one', 'two'), 'a')

    def test_nofile(self):
        # if a user tries to use an inexistant file in extensions,
        # pops an error
        self.assertRaises(IOError, Config, self.file_three)

    def test_load_into_settings(self):
        settings = {}
        load_into_settings(self.file_four, settings)
        self.assertEquals(settings["storage.e"], "f")
        self.assertEquals(settings["storage.g"], "h")
        self.assertEquals(settings["multi.once.storage.i"], "j")
        self.assertEquals(settings["multi.thrice.storage.i"], "jjj")

    def test_get_configurator(self):
        global_config = {"__file__": self.file_four}
        settings = {"pyramid.testing": "test"}
        config = get_configurator(global_config, **settings)
        settings = config.get_settings()
        self.assertEquals(settings["pyramid.testing"], "test")
        self.assertEquals(settings["storage.e"], "f")
        self.assertEquals(settings["storage.g"], "h")
        self.assertEquals(settings["multi.once.storage.i"], "j")
        self.assertEquals(settings.getsection("multi")["once.storage.i"], "j")
        self.assertEquals(settings.getsection("multi.once")["storage.i"], "j")
        self.assertEquals(settings.getsection("multi.once.storage")["i"], "j")

    def test_get_configurator_nofile(self):
        global_config = {"blah": "blech"}
        settings = {"pyramid.testing": "test"}
        config = get_configurator(global_config, **settings)
        settings = config.get_settings()
        self.assertEquals(settings["pyramid.testing"], "test")

    def test_settings_dict_copy(self):
        settings = SettingsDict({
          "a.one": 1,
          "a.two": 2,
          "b.three": 3,
          "four": 4,
        })
        new_settings = settings.copy()
        self.assertEqual(settings, new_settings)
        self.failUnless(isinstance(new_settings, SettingsDict))

    def test_settings_dict_getsection(self):
        settings = SettingsDict({
          "a.one": 1,
          "a.two": 2,
          "b.three": 3,
          "four": 4,
        })
        self.assertEquals(settings.getsection("a"), {"one": 1, "two": 2})
        self.assertEquals(settings.getsection("b"), {"three": 3})
        self.assertEquals(settings.getsection("c"), {})
        self.assertEquals(settings.getsection(""), {"four": 4})

    def test_settings_dict_setdefaults(self):
        settings = SettingsDict({
          "a.one": 1,
          "a.two": 2,
          "b.three": 3,
          "four": 4,
        })
        settings.setdefaults({"a.two": "TWO", "a.five": 5, "new": "key"})
        self.assertEquals(settings.getsection("a"),
                         {"one": 1, "two": 2, "five": 5})
        self.assertEquals(settings.getsection("b"), {"three": 3})
        self.assertEquals(settings.getsection("c"), {})
        self.assertEquals(settings.getsection(""), {"four": 4, "new": "key"})



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

import sys
import traceback
import simplejson as json

from pyramid.httpexceptions import WSGIHTTPException, HTTPServiceUnavailable

import mozsvc
from mozsvc.exceptions import BackendError
from mozsvc.middlewares import create_hash


def get_logger(request):
    """Get the logger object to use for the given request.

    This will return a metlog client if one is attached to the request,
    or the default mozsvc.logger object if not.
    """
    logger = request.registry.get("metlog")
    if logger is None:
        logger = mozsvc.logger
    return logger


def catch_backend_errors(handler, registry):
    """Tween to turn BackendError into a 503 response.

    This is a pyramid tween factory for catching BackendError exceptions
    and translating them into a HTTP "503 Service Unavailable" response.
    """
    def catch_backend_errors_tween(request):
        try:
            return handler(request)
        except BackendError as err:
            logger = get_logger(request)
            err_info = str(err)
            err_trace = traceback.format_exc()
            try:
                extra_info = "user: %s" % (request.user,)
            except Exception:
                extra_info = "user: -"
            error_log = "%s\n%s\n%s" % (err_info, err_trace, extra_info)
            hash = create_hash(error_log)
            logger.error(hash)
            logger.error(error_log)
            msg = json.dumps("application error: crash id %s" % hash)
            if err.retry_after is not None:
                if err.retry_after == 0:
                    retry_after = None
                else:
                    retry_after = err.retry_after
            else:
                settings = request.registry.settings
                retry_after = settings.get("mozsvc.retry_after", 1800)

            return HTTPServiceUnavailable(body=msg, retry_after=retry_after,
                                          content_type="application/json")

    return catch_backend_errors_tween


def log_uncaught_exceptions(handler, registry):
    """Tween to log all uncaught exceptions via metlog."""

    def log_uncaught_exceptions_tween(request):
        try:
            return handler(request)
        except WSGIHTTPException:
            raise
        except Exception:
            logger = get_logger(request)
            # We don't want to write arbitrary user-provided data into the
            # the logfiles.  For example, the sort of data that might show
            # up in the payload of a ValueError exception.
            # Format the traceback using standard printing, but use repr()
            # on the exception value itself to avoid this issue.
            exc_type, exc_val, exc_tb = sys.exc_info()
            lines = ["Uncaught exception while processing request:\n"]
            lines.append("%s %s\n" % (request.method, request.path_url))
            lines.extend(traceback.format_tb(exc_tb))
            lines.append("%r\n" % (exc_type,))
            lines.append("%r\n" % (exc_val,))
            logger.exception("".join(lines))
            raise

    return log_uncaught_exceptions_tween


def includeme(config):
    """Include all the mozsvc tweens into the given config."""
    config.add_tween("mozsvc.tweens.catch_backend_errors")
    config.add_tween("mozsvc.tweens.log_uncaught_exceptions")



# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****


import time
from decimal import Decimal, InvalidOperation

from pyramid.util import DottedNameResolver


def round_time(value=None, precision=2):
    """Transforms a timestamp into a two digits Decimal.

    Arg:
        value: timestamps representation - float or str.
        If None, uses time.time()

        precision: number of digits to keep. defaults to 2.

    Return:
        A Decimal two-digits instance.
    """
    if value is None:
        value = time.time()
    if not isinstance(value, str):
        value = str(value)
    try:
        digits = '0' * precision
        return Decimal(value).quantize(Decimal('1.' + digits))
    except InvalidOperation:
        raise ValueError(value)


def resolve_name(name, package=None):
    """Resolve dotted name into a python object.

    This function resolves a dotted name as a reference to a python object,
    returning whatever object happens to live at that path.  It's a simple
    convenience wrapper around pyramid's DottedNameResolver.

    The optional argument 'package' specifies the package name for relative
    imports.  If not specified, only absolute paths will be supported.
    """
    return DottedNameResolver(package).resolve(name)


def maybe_resolve_name(name_or_object, package=None):
    """Resolve dotted name or object into a python object.

    This function resolves a dotted name as a reference to a python object,
    returning whatever object happens to live at that path.  If the given
    name is not a string, it is returned unchanged.

    The optional argument 'package' specifies the package name for relative
    imports.  If not specified, only absolute paths will be supported.
    """
    return DottedNameResolver(package).maybe_resolve(name_or_object)



# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****


"""
Integration between mozsvc.user and pyramid_whoauth.
"""

from urlparse import urlparse

from zope.interface import implements

from repoze.who.interfaces import IAuthenticator
from repoze.who.plugins.macauth import MACAuthPlugin

from pyramid.threadlocal import get_current_request

import tokenlib

import mozsvc
import mozsvc.user
import mozsvc.secrets


class BackendAuthPlugin(object):
    """IAuthenticator plugin using mozsvc.user.authenticate().

    This is a repoze.who IAuthenticator plugin that checks user credentials
    against the mozsvc user database backend.  You must configure a backend
    for your application and should probably use mozsvc.user.RequestWithUser
    as your request factory.
    """
    implements(IAuthenticator)

    def authenticate(self, environ, identity):
        # We need a pyramid Request object to pass to the authenticator.
        # If mozsvc.user.RequestWithUser is being used as the request factory
        # then we can re-create the request object from the environ.  If not
        # then we will need to pull it out of the threadlocals.
        if "mozsvc.user.registry" in environ:
            request = mozsvc.user.RequestWithUser(environ)
        else:
            request = get_current_request()
        # Mark the user object as "under construction".
        # Otherwise when the call to mozsvc.user.authenticate() tries to
        # access request.user, it will recurse back into this function.
        environ.setdefault("mozsvc.user.temp_user", {})
        # Authenticate against the backend.  This has the side-effect of
        # setting identity["username"] to the authenticated username.
        if not mozsvc.user.authenticate(request, identity):
            return None
        return identity["username"]


class SagradaMACAuthPlugin(MACAuthPlugin):
    """MAC Auth plugin designed for use with Sagrada auth tokens.

    This is a repoze.who IIdentifier/IAuthenticator/IChallenger that
    consumes Sagrada authentication tokens as described here:

        https://wiki.mozilla.org/Services/Sagrada/TokenServer

    For verification of token signatures, this plugin can use either a
    single fixed secret (via the argument 'secret') or a file mapping
    node hostnames to secrets (via the argument 'secrets_file').  The
    two arguments are mutually exclusive.
    """

    def __init__(self, secret=None, secrets_file=None, **kwds):
        if secret is not None and secrets_file is not None:
            msg = "Can only specify one of 'secret' or 'secrets_file'"
            raise ValueError(msg)
        elif secret is None and  secrets_file is None:
            # Using secret=None will cause tokenlib to use a randomly-generated
            # secret.  This is useful for getting started without having to
            # twiddle any configuration files, but probably not what anyone
            # wants to use long-term.
            msgs = ["WARNING: using a randomly-generated token secret.",
                    "You probably want to set 'secret' or 'secrets_file' in"
                    "the [who.plugin.macauth] section of your configuration"]
            for msg in msgs:
                mozsvc.logger.warn(msg)
        if secrets_file is not None:
            self.secret = None
            self.secrets = mozsvc.secrets.Secrets(secrets_file)
        else:
            self.secret = secret
            self.secrets = None
        super(SagradaMACAuthPlugin, self).__init__(**kwds)

    def decode_mac_id(self, request, id):
        """Decode the MAC id into its secret key and dict of user data.

        This method determines the appropriate secrets to use for the given
        request, then passes them on to tokenlib to handle the given MAC id
        token.

        If the id is invalid then ValueError will be raised.
        """
        # There might be multiple secrets in use, if we're in the
        # process of transitioning from one to another.  Try each
        # until we find one that works.
        secrets = self._get_token_secrets(request)
        for secret in secrets:
            try:
                data = tokenlib.parse_token(id, secret=secret)
                key = tokenlib.get_token_secret(id, secret=secret)
                break
            except ValueError:
                pass
        else:
            raise ValueError("invalid MAC id")
        return key, data

    def encode_mac_id(self, request, data):
        """Encode the given data into a MAC id and secret key.

        This method is essentially the reverse of decode_mac_id.  It is
        not needed for consuming authentication tokens, but is very useful
        when building them for testing purposes.
        """
        # There might be multiple secrets in use, if we're in the
        # process of transitioning from one to another.  Always use
        # the last one aka the "most recent" secret.
        secret = self._get_token_secrets(request)[-1]
        id = tokenlib.make_token(data, secret=secret)
        key = tokenlib.get_token_secret(id, secret=secret)
        return id, key

    def _get_token_secrets(self, request):
        """Get the list of possible secrets for signing tokens."""
        if self.secrets is None:
            return [self.secret]
        # Secrets are looked up by hostname.
        # We need to normalize some port information for this work right.
        node_name = request.host_url
        host_url = urlparse(request.host_url)
        if host_url.scheme == "http" and host_url.port == 80:
            assert node_name.endswith(":80")
            node_name = node_name[:-3]
        elif host_url.scheme == "http" and host_url.port == 443:
            assert node_name.endswith(":443")
            node_name = node_name[:-4]
        return self.secrets.get(node_name)


def configure_who_defaults(config):
    """Configure default settings for authentication via repoze.who.

    This function takes a Configurator object using pyramid_whoauth and
    applies some default settings to authenticate against the configured
    user database backend.  Specifically:

        * add a plugin to identify/challenge/authenticate using Sagrada
          MAC Auth tokens.

        * if there is a configured authentication backend, add a plugin to
          authenticate against it.

        * if there is a configured authentication backend, add a plugin to
          identify/challenge using HTTP Basic Auth.

    All of these defaults may be overridden in the application config file.
    """
    settings = config.registry.settings
    BACKENDAUTH_DEFAULTS = {
        "use": "mozsvc.user.whoauth:BackendAuthPlugin"
    }
    for key, value in BACKENDAUTH_DEFAULTS.iteritems():
        settings.setdefault("who.plugin.backend." + key, value)
    BASICAUTH_DEFAULTS = {
        "use": "repoze.who.plugins.basicauth:make_plugin",
        "realm": "Sync",
    }
    for key, value in BASICAUTH_DEFAULTS.iteritems():
        settings.setdefault("who.plugin.basicauth." + key, value)
    MACAUTH_DEFAULTS = {
        "use": "mozsvc.user.whoauth:SagradaMACAuthPlugin",
    }
    for key, value in MACAUTH_DEFAULTS.iteritems():
        settings.setdefault("who.plugin.macauth." + key, value)
    # If there is an auth backend, enable basicauth by default.
    # Enable macauth by default regardless, since it doesn't need a backend.
    if config.registry.get("auth") is not None:
        settings.setdefault("who.authenticators.plugins", "backend macauth")
        settings.setdefault("who.identifiers.plugins", "basicauth macauth")
        settings.setdefault("who.challengers.plugins", "basicauth macauth")
    else:
        settings.setdefault("who.authenticators.plugins", "macauth")
        settings.setdefault("who.identifiers.plugins", "macauth")
        settings.setdefault("who.challengers.plugins", "macauth")


def includeme(config):
    # Make sure mozsvc.user is being used.
    # If already included then this is a no-op.
    config.include("mozsvc.user")

    # Set sensible default settings for whoauth.
    configure_who_defaults(config)

    # Use pyramid_whoauth for the authentication.
    config.include("pyramid_whoauth")



# ***** BEGIN LICENSE BLOCK *****
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
# ***** END LICENSE BLOCK *****
""" Cornice default views/.
"""
from pyramid.view import view_config


@view_config(route_name='heartbeat', renderer='string')
def hearbeat(request):
    return 'OK'
